#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>

int semid;

int lock_initialise(void)
{
  key_t key;
  union semun arg;
  
  if ((key = ftok(".lock", 'N')) == -1) {
    perror("ftok");
    return 0;
  }
  
  /* create a semaphore set with 1 semaphore: */
  if ((semid = semget(key, 1, 0666 | IPC_CREAT)) == -1) {
    perror("semget");
    return 0;
  }
  
  /* initialize semaphore #0 to 1: */
  arg.val = 1;
  if (semctl(semid, 0, SETVAL, arg) == -1) {
    perror("semctl");
    return 0;
  }
  
  return 1;
}

int lock_on(void)
{
  struct sembuf sb = {0, -1, SEM_UNDO};  /* set to allocate resource */

 retry:
  if (semop(semid, &sb, 1) == -1) {
    if (errno == EINTR) goto retry;
    perror("semop");
    return 0;
  }
  return 1;
}

int lock_off(void)
{
  struct sembuf sb = {0, 1, SEM_UNDO};  /* set to free resource */

 retry:
  if (semop(semid, &sb, 1) == -1) {
    if (errno == EINTR) goto retry;
    perror("semop");
    return 0;
  }
  return 1;
}

int lock_delete(void)
{
  union semun arg;
  
  /* remove semaphore */
  if (semctl(semid, 0, IPC_RMID, arg) == -1) {
    perror("semctl");
    return 0;
  }
  
  return 1;
}
